"use strict";
exports.__esModule = true;
var defaults_1 = require("./defaults");
var path_1 = require("path");
var fs_1 = require("fs");
var utils_1 = require("./utils");
var lodash_1 = require("lodash");
exports.outputProvider = function (options, store) {
    var outputFilter = options.outputFilter, dest = options.dest;
    var build = function build(pathname) {
        if (pathname === void 0) { pathname = ''; }
        pathname = utils_1.fixPath(pathname);
        var fn = defaults_1.thenFn();
        if (outputFilter(pathname) && dest) {
            fn = function (resolve, reject) {
                store.load(pathname).then(function (data) {
                    var absolutePathname = path_1.join(dest, pathname);
                    if (lodash_1.isPlainObject(data)) {
                        if (!fs_1.existsSync(absolutePathname)) {
                            fs_1.mkdirSync(absolutePathname);
                        }
                        Promise.all(Object.keys(data).map(function (file) { return build(path_1.join(pathname, file)); }))
                            .then(resolve)["catch"](defaults_1.catchFn(reject));
                    }
                    else {
                        fs_1.writeFile(absolutePathname, data, function (err) {
                            if (err) {
                                defaults_1.catchFn(reject)(err);
                            }
                            else {
                                resolve();
                            }
                        });
                    }
                });
            };
        }
        return new Promise(fn);
    };
    return build;
};
exports["default"] = exports.outputProvider;
